import { ApiPromise } from '@polkadot/api';
import { ISubmittableResult } from '@polkadot/types/types';
import { Signer } from 'ethers';
import Big, { RoundingMode } from 'big.js';

declare function getPolkadotApi(
  ws: string,
  onDisconnect?: VoidFunction,
): Promise<ApiPromise>;

declare type Hash = string;
declare type ExtrinsicEventsCallback = (event: ExtrinsicEvent) => void;
declare type ExtrinsicEvent =
  | ExtrinsicFailedEvent
  | ExtrinsicSentEvent
  | ExtrinsicSuccessEvent;
interface ExtrinsicFailedEvent {
  blockHash: Hash;
  message?: string;
  status: ExtrinsicStatus.Failed;
  txHash: Hash;
}
interface ExtrinsicSentEvent {
  status: ExtrinsicStatus.Sent;
  txHash: Hash;
}
interface ExtrinsicSuccessEvent {
  blockHash: Hash;
  status: ExtrinsicStatus.Success;
  txHash: Hash;
}
declare enum ExtrinsicStatus {
  Failed = 'Failed',
  Sent = 'Sent',
  Success = 'Success',
}

declare function createExtrinsicEventHandler(
  pallet: string,
  successEvent: string,
  cb: ExtrinsicEventsCallback,
): ({ events, status, txHash }: ISubmittableResult) => void;

declare function createTxEventHandler(
  ethersSigner: Signer,
  txHash: Hash,
  cb: ExtrinsicEventsCallback,
  skipSentEvent?: boolean,
): Promise<void>;

declare function toDecimal(
  number: bigint | number | string,
  decimals: number,
  maxDecimal?: number,
  roundType?: RoundingMode,
): Big;
declare function toBigInt(amount: string | number, decimals: number): bigint;
declare function hasDecimalOverflow(
  fl: number | string,
  maxDecimal: number,
): boolean;

export {
  ExtrinsicEvent,
  ExtrinsicEventsCallback,
  ExtrinsicFailedEvent,
  ExtrinsicSentEvent,
  ExtrinsicStatus,
  ExtrinsicSuccessEvent,
  Hash,
  createExtrinsicEventHandler,
  createTxEventHandler,
  getPolkadotApi,
  hasDecimalOverflow,
  toBigInt,
  toDecimal,
};
